import { Client } from "smtp-fetch";
import { SMTPResult } from "../types";
import { TcpNetConnectOpts } from "net";

async function checkSMTP(
  to: string,
  host: string,
  port: number,
  {fromAddr, timeout = 5000}: {timeout?: number, fromAddr?: string} = {},
): Promise<SMTPResult> {
  const c = new Client(host, port);

  try {
    const options: Omit<TcpNetConnectOpts, 'host' | 'port'> = { timeout, localAddress: fromAddr };
    await c.connect(options);

    await c.mail("");
    await c.rcpt(to);
    await c.quit();

    return {
      valid: true,
    };
  } catch (err: any) {
    return {
      valid: false,
      error: err.message,
    };
  } finally {
    c.close();
  }
}

export default checkSMTP;
